# Generated by Django 2.0.2 on 2018-02-12 23:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('todo', '0004_rename_list_tasklist'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='tasklist',
            options={'ordering': ['name'], 'verbose_name_plural': 'Task Lists'},
        ),
        migrations.AlterField(
            model_name='item',
            name='completed',
            field=models.BooleanField(default=False),
        ),
    ]
