{% extends "base.html" %}
{% load staticfiles %}

{% block extrahead %}
	<!-- CSS and JavaScripts for django-todo -->
	<link rel="stylesheet" type="text/css" href="{% static 'todo/css/styles.css' %}" />
{% endblock extrahead %}
