# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-09 11:11
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('todo', '0002_auto_20150614_2339'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='assigned_to',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='todo_assigned_to', to=settings.AUTH_USER_MODEL),
        ),
    ]
